use async_std::task::block_on;
use sfsm::State;

use crate::{
  game::{
    stage::{Stage, StageNumber},
    GameContext,
  },
  utils::game::Game,
};

use super::{loading::Loading, title_mof::TitleScreenMof, UPDATE_CONTEXT};

pub struct Gameplay {
  stage: Stage,
}

impl Gameplay {
  pub fn new_blocking(context: &mut GameContext<'_>, game: Game, stage_number: StageNumber) -> Self {
    block_on(Self::new(context, game, stage_number))
  }
  pub async fn new(context: &mut GameContext<'_>, game: Game, stage_number: StageNumber) -> Self {
    let stage = Stage::new(context, game, stage_number).await;
    Self { stage }
  }
}

impl State for Gameplay {
  fn entry(&mut self) {
    UPDATE_CONTEXT.with(|context| self.stage.started(context));
  }
  fn execute(&mut self) {
    UPDATE_CONTEXT.with(|context| self.stage.update(context));
  }
}

impl From<Loading> for Gameplay {
  fn from(_: Loading) -> Self {
    UPDATE_CONTEXT.with(|context| Gameplay::new_blocking(context, Game::Mof, StageNumber::Stage1))
  }
}

impl From<TitleScreenMof> for Gameplay {
  fn from(_: TitleScreenMof) -> Self {
    UPDATE_CONTEXT.with(|context| Gameplay::new_blocking(context, Game::Mof, StageNumber::Stage1))
  }
}
