mod main_menu;

use std::sync::Arc;

use main_menu::MainMenu;
use sfsm::*;
use winit::keyboard::KeyCode;

use crate::{
  game::{
    anm::{LoadedFile, Vm, VmLocation},
    GameContext,
  },
  utils::game::Game,
};

use super::{gameplay::Gameplay, loading::Loading, UPDATE_CONTEXT};

pub struct TitleScreenMof {
  _splash: Vm,
  _title_anm: Arc<LoadedFile>,
  machine: TitleScreenMachine,
}

add_state_machine!(TitleScreenMachine, MainMenu, { MainMenu }, {});

impl TitleScreenMof {
  fn new(context: &mut GameContext, _: bool) -> Self {
    let title = context.load_anm(Game::Mof, "title.anm");

    // 79, 83
    let splash = context.anm_manager.new_vm(title.clone(), None, 88, VmLocation::new_ui());

    Self {
      _splash: splash,
      _title_anm: title,
      machine: TitleScreenMachine::new(),
    }
  }
}

impl State for TitleScreenMof {
  fn entry(&mut self) {
    self.machine.start(MainMenu::new()).unwrap();
  }
  fn execute(&mut self) {
    self.machine.step().unwrap();
  }
}

impl From<Loading> for TitleScreenMof {
  fn from(_: Loading) -> Self {
    UPDATE_CONTEXT.with(|context| {
      context.sound_manager.get_bgm_manager(Game::Mof).play_track("th10_02.wav");
      // context.sound_manager.get_bgm_manager(Game::Mof).play_track("th11_00.wav");
      TitleScreenMof::new(context, true)
    })
  }
}

impl Transition<Gameplay> for TitleScreenMof {
  fn guard(&self) -> TransitGuard {
    match &self.machine.states {
      TitleScreenMachineStates::MainMenuState(Some(main_menu)) => main_menu.switch_to_gameplay().into(),
      _ => TransitGuard::Remain
    }
  }

  fn action(&mut self) {
    println!("hsttkasdgn")
  }
}
