use sfsm::{State, TransitGuard, Transition};
use winit::keyboard::KeyCode;

use crate::{
  game::{
    anm::{Vm, VmLocation},
    GameContext,
  },
  utils::game::Game,
};

use super::{loading::Loading, UPDATE_CONTEXT};

pub struct TitleScreenSa {
  _logo: Vm,
  _splash: Vm,
}

impl TitleScreenSa {
  fn new(context: &mut GameContext, _: bool) -> Self {
    let title = context.load_anm(Game::Sa, "title.anm");

    // 79, 83
    let splash = context.anm_manager.new_vm(title.clone(), Some(8), 79, VmLocation::new_ui());
    let logo = context.anm_manager.new_vm(title.clone(), Some(8), 83, VmLocation::new_ui());

    Self {
      _logo: logo,
      _splash: splash,
    }
  }
}

impl State for TitleScreenSa {
  fn execute(&mut self) {
    UPDATE_CONTEXT.with(|context| {
      if context.engine.keys.was_key_pressed(KeyCode::Space) {
        self._logo.borrow_mut().interrupt(1);
      }
    })
  }
}

impl From<Loading> for TitleScreenSa {
  fn from(_: Loading) -> Self {
    UPDATE_CONTEXT.with(|context| {
      // context.sound_manager.get_bgm_manager(Game::Mof).play_track("th10_02.wav");
      context.sound_manager.get_bgm_manager(Game::Sa).play_track("th11_00.wav");
      TitleScreenSa::new(context, true)
    })
  }
}

impl Transition<TitleScreenSa> for TitleScreenSa {
  fn action(&mut self) {
    *self = UPDATE_CONTEXT.with(|context| TitleScreenSa::new(context, false))
  }

  fn guard(&self) -> TransitGuard {
    UPDATE_CONTEXT.with(|context| context.engine.keys.was_key_pressed(KeyCode::Space)).into()
    // UPDATE_CONTEXT.with(|_| self._logo.borrow().deleted()).into()
  }
}
